﻿using System;
using System.ComponentModel;
using System.Windows.Input;
using System.Collections.ObjectModel;
using Microsoft.Maps.MapControl.WPF;
using BingMapsCalculateRouteDemo.Core;
using System.Text.RegularExpressions;
using BingMapsCalculateRouteDemo.Business;

namespace BingMapsCalculateRouteDemo.ViewModels
{
    public class RouteViewModel: INotifyPropertyChanged
    {
        public ICommand CalculateRouteCommand { get; private set; }

        private string _from = "Meridian, ID";
        public string From
        {
            get { return _from; }
            set
            {
                _from = value;
                OnPropertyChanged("From");
            }
        }

        private string _to = "Boise, ID";
        public string To
        {
            get { return _to; }
            set
            {
                _to = value;
                OnPropertyChanged("To");
            }
        }

        private BingServices.RouteResult _routeResult;
        public BingServices.RouteResult RouteResult
        {
            get { return _routeResult; }
            set
            {
                _routeResult = value;
                OnPropertyChanged("RouteResult");
            }
        }
        

        private ObservableCollection<Direction> _directions;
        public ObservableCollection<Direction> Directions
        {
            get { return _directions; }
            set
            {
                _directions = value;
                OnPropertyChanged("Directions");
            }
        }

        public RouteViewModel()
        {
            CalculateRouteCommand = new DelegateCommand(CalculateRoute);
        }

        private void CalculateRoute()
        {
            var from = GeocodeAddress(From);
            var to = GeocodeAddress(To);

            CalculateRoute(from, to);
        }

        private BingServices.GeocodeResult GeocodeAddress(string address)
        {
            BingServices.GeocodeResult result = null;

            using (BingServices.GeocodeServiceClient client = new BingServices.GeocodeServiceClient("CustomBinding_IGeocodeService"))
            {
                BingServices.GeocodeRequest request = new BingServices.GeocodeRequest();
                request.Credentials = new Credentials() { ApplicationId = (App.Current.Resources["MyCredentials"] as ApplicationIdCredentialsProvider).ApplicationId };
                request.Query = address;
                result = client.Geocode(request).Results[0];
            }

            return result;
        }

        private void CalculateRoute(BingServices.GeocodeResult from, BingServices.GeocodeResult to)
        {
            using (BingServices.RouteServiceClient client = new BingServices.RouteServiceClient("CustomBinding_IRouteService"))
            {
                BingServices.RouteRequest request = new BingServices.RouteRequest();
                request.Credentials = new Credentials() { ApplicationId = (App.Current.Resources["MyCredentials"] as ApplicationIdCredentialsProvider).ApplicationId };
                request.Waypoints = new ObservableCollection<BingServices.Waypoint>();
                request.Waypoints.Add(ConvertResultToWayPoint(from));
                request.Waypoints.Add(ConvertResultToWayPoint(to));

                request.Options = new BingServices.RouteOptions();
                request.Options.RoutePathType = BingServices.RoutePathType.Points;

                RouteResult = client.CalculateRoute(request).Result;
            }

            GetDirections();
        }

        private void GetDirections()
        {
            Directions = new ObservableCollection<Direction>();

            foreach (BingServices.ItineraryItem item in RouteResult.Legs[0].Itinerary)
            {
                var direction = new Direction();
                direction.Description = GetDirectionText(item);
                direction.Location = new Location(item.Location.Latitude, item.Location.Longitude);
                Directions.Add(direction);
            }
        }

        private static string GetDirectionText(BingServices.ItineraryItem item)
        {
            string contentString = item.Text;
            //Remove tags from the string
            Regex regex = new Regex("<(.|\n)*?>");
            contentString = regex.Replace(contentString, string.Empty);
            return contentString;
        }

        private BingServices.Waypoint ConvertResultToWayPoint(BingServices.GeocodeResult result)
        {
            BingServices.Waypoint waypoint = new BingServices.Waypoint();
            waypoint.Description = result.DisplayName;
            waypoint.Location = result.Locations[0];
            return waypoint;
        }

        public event PropertyChangedEventHandler PropertyChanged;
        protected virtual void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }
    }
}